@@ -3,6 +3,9 @@ from fixieai.agents.api import AgentResponse
 from fixieai.agents.api import Embed
 from fixieai.agents.api import Message
 from fixieai.agents.code_shot import CodeShotAgent
+from fixieai.agents.oauth import OAuthHandler
+from fixieai.agents.oauth import OAuthParams
+from fixieai.agents.user_storage import UserStorage
 
 __all__ = [
     "Message",
@@ -10,4 +13,7 @@ __all__ = [
     "AgentQuery",
     "AgentResponse",
     "CodeShotAgent",
+    "OAuthHandler",
+    "OAuthParams",
+    "UserStorage",
 ]
