@@ -102,9 +102,11 @@ class CodeShotAgent:
         self.few_shots = few_shots
         self.oauth_params = oauth_params
         self._funcs: Dict[str, Callable] = {}
+        self._jwks_client = jwt.PyJWKClient(constants.FIXIE_JWKS_URL)
 
-        # Register default Funcs.
-        self.register_func(_oauth)
+        if oauth_params is not None:
+            # Register default Funcs.
+            self.register_func(_oauth)
 
     def serve(self, host: str = "0.0.0.0", port: int = 8181):
         """Starts serving the current agent at `{host}:{port}` via uvicorn.
@@ -211,7 +213,13 @@ class CodeShotAgent:
 
     def _verify_token(self, token: str) -> bool:
         try:
-            _ = jwt.decode(token, constants.FIXIE_PUBLIC_KEY, algorithms=["EdDSA"])
+            public_key = self._jwks_client.get_signing_key_from_jwt(token)
+            _ = jwt.decode(
+                token,
+                public_key.key,
+                algorithms=["EdDSA"],
+                audience=constants.FIXIE_AGENT_API_AUDIENCE,
+            )
             return True
         except jwt.DecodeError:
             return False
