@@ -15,8 +15,8 @@ FIXIE_REFRESH_URL = f"{FIXIE_API_URL}/api/refresh"
 # Fixie's OAuth redirect endpoint. Tokens sent to this endpoint will be redirected back
 # to your agent.
 FIXIE_OAUTH_REDIRECT_URL = f"{FIXIE_API_URL}/oauth"
+# Fixie's JWKS URL.
+FIXIE_JWKS_URL = f"{FIXIE_API_URL}/.well-known/jwks.json"
 
-
-FIXIE_PUBLIC_KEY = """-----BEGIN PUBLIC KEY-----
-MCowBQYDK2VwAyEANcDOXX9KOKx64wNUuq9oyGKfj5lZJjM/0Qgj/A55PTw=
------END PUBLIC KEY-----"""
+# The audience for Fixie's query JWTs.
+FIXIE_AGENT_API_AUDIENCE = "https://app.fixie.ai/api"
